/*--------------------------------------------------------------------------*/
/*  (C) Copyright Monstein Radio Astronomy Support, Freienbach Switzerland  */
/*--------------------------------------------------------------------------*/
/*  Programmname: mainunit.cpp                                              */
/*                                                                          */
/*  Revision: V1.0     Date: 18.06.2004    Autor: Chr. Monstein             */
/*                                                                          */
/*  Purpose: Hostcontroller satellite rotor                                 */
/*                                                                          */
/*  Compiler: BORLAND C++Builder 6                                          */
/*--------------------------------------------------------------------------*/

// Created by: Chr. Monstein 15.06.2004
// Updated by: Chr. Monstein 18.06.2004  Button Set TimeDate
// Updated by: Rafael Spoerri 20.09.2012  V1.0 Twarm insert
// Updated by: Chr. Monstein 03.02.2015
// Updated by: Chr. Monstein 06.07.2016  add logfile
// Updated by: Chr. Monstein 05.08.2016  new firmware
// Updated by: Chr. Monstein 23.09.2022  update for DiSEqC controller
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#define WIN32
#include "mainunit.h"
#include "WSC.H"
#include "ClinoRS232.H"

#include <stdio.h>
#include <fstream.h>
#include <math.h> // pow

#pragma resource "*.dfm"
//---------------------------------------------------------------------------
// import form ClinoRS232.cpp
extern char comport[40];
extern TWrite *Clino_SerialThread; // Arduino activities
extern char ringbuff[buflen];
extern int in, out;
//---------------------------------------------------------------------------
float azi;
float ele;
float eleref;
float aziref;
float maxrange;

AnsiString cmd;   // manual commands to _Arduino

Word oldday=99999, PDay, PYear, PMonth;
bool logfirst = false;

TMainForm *MainForm;   // mainunit of Borland C-Builder Program

//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
   : TForm(Owner)
{
    ReadConfig();

    Clino_SerialThread = new TWrite(false);
    Clino_SerialThread->Init(); // Connect ARDUINO-interface Port

    Sleep(200);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BitBtn1Click(TObject *Sender)   //Exit
{
    Application->Terminate();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
    Clino_SerialThread->Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EvalArduino(void)  // Evaluate Arduino data
{
    char message[80]="\0";
    char tmp[20];
    static int i = 0;
    char c;
    int dp; // difference of buffer pointers

    dp = abs(in-out); // difference pointer
    if (dp > 0)
    {
        do
        {
           c = (ringbuff[out] & 127);   // just ASCII code
           message[i++]=c;
           out=(out+1)%buflen;
           if (in==out) break; // buffer is now empty
        } while (1);
        Memo1->Lines->Add(message);

        // check terminator character....
        /*
        if (i>2)
        {
          message[i  ]='\0';
          message[i-1]='\0';
          message[i-2]='\0';
        }  */
        i = 0; // delete ringbuffer

        message[i]='\0';
        sprintf(tmp,"Pin=%u Pout=%u",in,out);
        MainForm->Label3->Caption=tmp; // show buffer pointers
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TimerTimer(TObject *Sender) // some periodic activity
{
    Word hour,  min,  sec,  msec, Year, Month, Day;
    AnsiString Timer;

    Timer = TimeToStr(Time());
    DecodeTime(Timer, hour, min, sec, msec);
    Label1->Caption = "PC-time: "+Timer;

    Timer = DateToStr(Date());
    DecodeDate(Timer,Year, Month, Day);
    Label9->Caption = "PC-date: "+Timer;

    EvalArduino(); // read Arduino buffer and analyze data
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ReadConfig(void)
{
  char buff[120];
  AnsiString st;
  float timer_sec;
  ifstream infile("params.cfg");
  while (infile)
  {
      infile.getline(buff,120);
      st = LowerCase(AnsiString(buff));
      if (strstr(st.c_str(),"[comport]="))   sscanf(buff,"[comport]=%s",  &comport);
      if (strstr(st.c_str(),"[aziref]="))    sscanf(buff,"[aziref]=%f",   &aziref);
      if (strstr(st.c_str(),"[azimuth]="))   sscanf(buff,"[azimuth]=%f",  &azi);
      if (strstr(st.c_str(),"[elevation]=")) sscanf(buff,"[elevation]=%f",&ele);
      if (strstr(st.c_str(),"[eleref]="))    sscanf(buff,"[eleref]=%f",   &eleref);
      if (strstr(st.c_str(),"[maxrange]="))  sscanf(buff,"[maxrange]=%f", &maxrange);
  }
  infile.close();
  Edit1->Text = azi;
  Edit3->Text = aziref;
  Edit2->Text = ele;
  Edit4->Text = eleref;
  
  sprintf(buff,"Current maximum range for both rotors:%5.1f\r",maxrange);
  Memo1->Lines->Add(buff);
  Memo1->Lines->Add("Value can be changed in file params.cfg\r");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Button1Click(TObject *Sender)
{
   Clino_SerialThread->Send("-v\r");
   //Label8->Caption="Version?";
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Button3Click(TObject *Sender)
{
   Memo1->Lines->Add("Button Commands\r");
   Memo1->Lines->Add("Button help = shows all the commands\r");
   Memo1->Lines->Add("Button Version = version of the uploaded Arduino program\r");
   Memo1->Lines->Add("Button Exit = leave the program\r");
   Memo1->Lines->Add("\r");
   Memo1->Lines->Add("Elevation with respect to Earth\r");
   Memo1->Lines->Add("EleReference = mechanical zero-position of rotor\r");
   Memo1->Lines->Add("Elevation rotor angle = Elevation - EleReference\r");
   Memo1->Lines->Add("\r");
   Memo1->Lines->Add("Azimuth with respect to Earth\r");
   Memo1->Lines->Add("AziReference = mechanical zero-position of rotor (south=180)\r");
   Memo1->Lines->Add("Azimuth rotor angle = Azimuth - AziReference\r");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Button2Click(TObject *Sender)   // set azimuth
{
    char tmp[20];
    cmd = Edit1->Text;
    azi = atof(cmd.c_str());
    cmd = Edit3->Text;
    aziref = atof(cmd.c_str());
    azi = azi - aziref;
    if (CheckBox1->Checked)
       azi = -azi;
    sprintf(tmp,"azi%6.2f\r",azi);
    Clino_SerialThread->Send(tmp);
    sprintf(tmp,"Azimuth rotor: %6.2f",azi);
    Label8->Caption = tmp;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Button4Click(TObject *Sender)  // set elevation
{
    char tmp[20];
    cmd = Edit2->Text;
    ele = atof(cmd.c_str());
    cmd = Edit4->Text;
    eleref = atof(cmd.c_str());
    ele = ele - eleref;
    if (CheckBox2->Checked)
       ele = -ele;
    sprintf(tmp,"ele%6.2f\r",ele);
    Clino_SerialThread->Send(tmp);
    sprintf(tmp,"Elevation rotor: %6.2f",ele);
    Label12->Caption = tmp;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Button5Click(TObject *Sender)
{
   ReadConfig();
}
//---------------------------------------------------------------------------


